#include <math.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include "gtk/gtk.h"
#include "libgimp/gimp.h"
#include "libgimp/gimpui.h"
#include "effects.h"


void rgb_to_hsv (double red, double green, double blue,
		 double *hue, double *saturation, double *value)
{
  double h, s, v;
  double min, max;
  double delta;

  h = 0.0; /* Shut up -Wall */

  if (red > green)
    {
      if (red > blue)
        max = red;
      else
        max = blue;

      if (green < blue)
        min = green;
      else
        min = blue;
    }
  else
    {
      if (green > blue)
        max = green;
      else
        max = blue;

      if (red < blue)
        min = red;
      else
        min = blue;
    }

  v = max;

  if (max != 0.0)
    s = (max - min) / max;
  else
    s = 0.0;

  if (s == 0.0)
    {
      h = 0.0;
    }
  else
    {
      delta = max - min;

      if (red == max)
        h = (green - blue) / delta;
      else if (green == max)
        h = 2 + (blue - red) / delta;
      else if (blue == max)
        h = 4 + (red - green) / delta;

      h /= 6.0;

      if (h < 0.0)
        h += 1.0;
      else if (h >= 1.0)
        h -= 1.0;
    }

  *hue        = h;
  *saturation = s;
  *value      = v;
}


void hsv_to_rgb (double hue, double saturation, double value, 
		 double *red, double *green, double *blue)
{
  double f, p, q, t;

  if (saturation == 0.0)
    {
      *red   = value;
      *green = value;
      *blue  = value;
    }
  else
    {
      hue *= 6.0;

      if (hue == 6.0)
        hue = 0.0;

      f = hue - (int) hue;
      p = value * (1.0 - saturation);
      q = value * (1.0 - saturation * f);
      t = value * (1.0 - saturation * (1.0 - f));

      switch ((int) hue)
        {
        case 0:
          *red = value;
          *green = t;
          *blue = p;
          break;

        case 1:
          *red = q;
          *green = value;
          *blue = p;
          break;

        case 2:
          *red = p;
          *green = value;
          *blue = t;
          break;

        case 3:
          *red = p;
          *green = q;
          *blue = value;
          break;
        case 4:
          *red = t;
          *green = p;
          *blue = value;
          break;

        case 5:
          *red = value;
          *green = p;
          *blue = q;
          break;
        }
    }
}

double spread(double x)
{
  return cos(x * M_PI) * -0.5 + 0.5;
}


void rainbow(gdouble *parameters,
	     int width, int height, int bpp,
	     guchar *srcpix, guchar *dstpix)
{
  int pixsize = height * width * bpp;
  int i;

  for (i = 0; i < pixsize; i += bpp)
  {
    guchar r,g,b;
    double red, green, blue;
    double hue, saturation, value;

    r = srcpix[i];
    g = srcpix[i+1];
    b = srcpix[i+2];

    red   = r / 255.0;
    green = g / 255.0;
    blue  = b / 255.0;

    rgb_to_hsv(red, green, blue, &hue, &saturation, &value);

    hue = value * (parameters[1] + 1.0) + parameters[0];
    hue = hue - (int) hue;
    //value = 1.0;
    //saturation = 1.0;

    hsv_to_rgb(hue, saturation, value, &red, &green, &blue);

      
    r = red * 255;
    g = green * 255;
    b = blue * 255;

    dstpix[i]   = r;
    dstpix[i+1] = g;
    dstpix[i+2] = b;
  }

}



void rubbing(gdouble *parameters,
	     int width, int height, int bpp,
	     guchar *srcpix, guchar *dstpix)

     /* the code herein is vastly inefficient.  sue me. */

#define XY(X,Y) (((X) + (Y) * width) * bpp)

{
  guchar *origpix;
  int pixsize = height * width * bpp;
  int x,y;
  int delta;

  if (parameters[0] != 0)
    delta = parameters[0];
  else
    delta = 5;

  for (x = 0; x < width; x++)
    for (y = 0; y < height; y++)
      {
	int x2, y2;

	int count, total, average, orig;
	int color;
	
	total = 0;
	count = 0;

	for (x2 = -delta; x2 <= delta; x2++)
	  for (y2 = -delta; y2 <= delta; y2++)
	    {
	      if (((x + x2) >= 0) &&
		  ((x + x2) < width) &&
		  ((y + y2) >= 0) &&
		  ((y + y2) < height))
		{
		  total += 
		    srcpix[XY(x + x2, y + y2)] + /* red */
		    srcpix[XY(x + x2, y + y2) + 1] + /* green */
		    srcpix[XY(x + x2, y + y2) + 2]; /* blue */
		  count++;
		}
	    }
	
	average = total / count;
	orig = 
	  srcpix[XY(x,y)] +
	  srcpix[XY(x,y)+1] +
	  srcpix[XY(x,y)+2];

	if (average > orig)
	  color = 0;
	else
	color = 255;

	dstpix[XY(x,y)]   = color;
	dstpix[XY(x,y)+1] = color;
	dstpix[XY(x,y)+2] = color;
	
      }

}
